<?php
namespace Teknisa\SM\Controller;

use Teknisa\SM\Controller\SMCrud;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\Libs\Util\NewCode;
use Teknisa\SM\Service\Sm00002000ModeloPropostaComposicaoService;

class Sm00002000ModeloPropostaComposicaoController extends SMCrud
{
    protected $dataSourceName = 'fct_relorcamodcomposicao_widget';
    protected $newCode;
    protected $ServiceComposicaoProp;

    public function __construct ($crudService, $environment, NewCode $newCode, Sm00002000ModeloPropostaComposicaoService $ServiceComposicaoProp) {
        parent::__construct($crudService, $environment);
        $this->newCode = $newCode;
        $this->ServiceComposicaoProp = $ServiceComposicaoProp;
    }

    public function save(Request $request, Response $response) {
        try {
            $rows = $request->getDataSet()->getRows();
            $orcaModelo = $rows[0]['NRORCAMENTOMODELO'];
            
            $codComposicoes = array();
            foreach ($rows as $row) {
                array_push($codComposicoes, $row["NRCOMPSERVICO"]);
            }
            
            $codComposicoesInseridas = $this->ServiceComposicaoProp->validaCompProposta($codComposicoes, $orcaModelo);
            
            $linhas = array();
            
            for ($i = 0; $i < count($rows); $i++) {
                if (!in_array($rows[$i]['NRCOMPSERVICO'], $codComposicoesInseridas)) {
                    $rows[$i]['NRRELORCAMODCOMP'] = $this->newCode->getCodeByTableName("FCT_RELORCAMODCOMPOSICAO", $this->environment->getNrOrg(), 1, 10, true);
                    array_push($linhas, $rows[$i]);
                }
            }
    
            $requestNovo = new Request\DataSet(new DataSet($this->dataSourceName, $linhas), $request->getRoutePath(), $request->getMethod(), $request->getUserId());
            parent::save($requestNovo, $response);
            $erro = $response->getError();
            if ($erro) {
                throw new \Exception($erro->getMessage(), $erro->getErrorCode());
            }
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function delete(Request $request, Response $response) {
        $row = $request->getRow();
        $nrorcamentomodelo = $row["NRORCAMENTOMODELO"];
        $nrcompservico     = $row["NRCOMPSERVICO"];
        $nrorg             = $row["NRORG"];
        
        $this->ServiceComposicaoProp->excluirEmCascata($nrorcamentomodelo, $nrcompservico, $nrorg);
        $response->addMessage(new Response\Message("Registro excluído com sucesso."));
    }
    
    public function find(Request\Filter $request, Response $response) {
    
        $request->setParameter('dbname', 'fct_relorcamodcomposicao_widget');
        parent::find($request, $response);
    
    }
}

?>