/* Sistemas Integrados TEKNISA
  Controle de Produo

Nome: 3912_CDPSTP0.SQL

Descrio:
Gera os dados para a tela MRP (Planejamento).

P_CDFILIAL     - Cdigo da Filial a ser gerado o MRP.
P_DTMRP        - Data da Gerao do MRP.
P_DTINI        - Data Inicial.
P_DTFIN        - Data Final.
P_CDPRODINI    - Cdigo do Produto Inicial.
P_CDPRODFIN    - Cdigo do Produto Final.
P_IDMRPPEDPREV - Identificador da gerao do MRP para Pedidos (P) e/ou
                 para Previso de Vendas (V) ou Plano de Produo (L).
P_STLISTPEDIDO - Lista de itens de Pedidos escolhidos (Plano de Produo).
P_STTODOPEDIDO - Todos os pedidos.
P_IDESTOQUE    - Identificador (considera ou no estoque).
P_IDCONSDEMA   - Identificador (considera ou no demanda extra).
P_ORCDE        - Numero inicial do oramento
P_ORCATE       - Numero Final  do oramento
P_LORC         - Lista de Orcamentos
P_CDDEPTO       - Codigo do depto

PARA DEBUGAR
-- SET SERVEROUTPUT ON
-- DBMS_OUTPUT.PUT_LINE('xx');

Criao:
15/09/2004 - Fabrcio Marcelo da Silva

DECLARE @V_DATAINI DATETIME;
SET @V_DATAINI = convert(datetime, '22/03/2001', 103);
DECLARE @V_DATAFIN DATETIME;
SET @V_DATAFIN = convert(datetime, '22/03/2010', 103);
DECLARE @V_HOJE DATETIME;
SET @V_HOJE = GETDATE();
exec GERA_PLANEJ_MRP '9999', @V_HOJE, @V_DATAINI, @V_DATAFIN, '01', '04zzzzzzzz', 'L', '_', NULL, 'N', 'S';

*/

IF EXISTS(SELECT * FROM sysobjects O, sysusers U
          WHERE
            o.name = 'GERA_PLANEJ_MRP'  and
            type = 'P'                  and
            u.uid = o.uid               and
            u.name = USER)
  DROP PROCEDURE GERA_PLANEJ_MRP
  GO

CREATE PROCEDURE GERA_PLANEJ_MRP
   @P_CDFILIAL       VARCHAR(4),
   @P_DTMRP          DATETIME,
   @P_DTINI          DATETIME,
   @P_DTFIN          DATETIME,
   @P_CDPRODINI      VARCHAR(20),
   @P_CDPRODFIN      VARCHAR(20),
   @P_IDMRPPEDPREV   VARCHAR(1),
   @P_STLISTPEDIDO   VARCHAR(10),
   @P_STTODOPEDIDO   VARCHAR(1),
   @P_IDESTOQUE      VARCHAR(1),
   @P_IDCONSDEMA     VARCHAR(1),
   @P_ORCDE          VARCHAR(10),
   @P_ORCATE         VARCHAR(10),
   @P_LORC           VARCHAR(2000),
   @P_CDDEPTO        VARCHAR(2000),
   @P_IDGERSEMIACA   VARCHAR(1),
   @P_STLISTAFILI    VARCHAR(2000),
   @P_STTFILISOLI    VARCHAR(1),
   @P_STLISTASOLI    VARCHAR(10),
   @P_STTSOLICITA    VARCHAR(1)
AS
   /* Variveis Matria Prima */
   DECLARE @V_CDPRODPREVMP   VARCHAR(10);
   DECLARE @V_QTCOMPRAS      DOUBLE PRECISION;

   DECLARE @C_PRODPED_CDPRODUTO     VARCHAR(10);
   DECLARE @C_PRODPED_IDPRODPROD    VARCHAR(1);
   DECLARE @C_PRODPED_DTMINENTPED   DATETIME;
   DECLARE @C_PRODPED_QTDEESTO      DOUBLE PRECISION;
   DECLARE @C_PRODPED_QTVENDAPCP    DOUBLE PRECISION;
   DECLARE @C_PRODPED_QTESTOQMIN    DOUBLE PRECISION;
   DECLARE @C_PRODPED_QTPADPROCFAB  DOUBLE PRECISION;
   DECLARE @C_PRODPED_QTLOTEFABDIA  DOUBLE PRECISION;
   DECLARE @C_PRODPED_QTPRODUCAO    DOUBLE PRECISION;
   DECLARE @C_PRODPED_QTANDPROD     DOUBLE PRECISION;
   DECLARE @C_PRODPED_QTTOTALPROD   DOUBLE PRECISION;

   DECLARE @C_PRODPLAN_CDPRODUTO     VARCHAR(10);
   DECLARE @C_PRODPLAN_IDPRODPROD    VARCHAR(1);
   DECLARE @C_PRODPLAN_DTMINENTPED   DATETIME;
   DECLARE @C_PRODPLAN_QTDEESTO      DOUBLE PRECISION;
   DECLARE @C_PRODPLAN_QTVENDAPCP    DOUBLE PRECISION;
   DECLARE @C_PRODPLAN_QTESTOQMIN    DOUBLE PRECISION;
   DECLARE @C_PRODPLAN_QTPADPROCFAB  DOUBLE PRECISION;
   DECLARE @C_PRODPLAN_QTLOTEFABDIA  DOUBLE PRECISION;
   DECLARE @C_PRODPLAN_QTPRODUCAO    DOUBLE PRECISION;
   DECLARE @C_PRODPLAN_QTANDPROD     DOUBLE PRECISION;
   DECLARE @C_PRODPLAN_QTTOTALPROD   DOUBLE PRECISION;

   DECLARE @C_MATEPRIMA_CDPRODINSUMO VARCHAR(10);
   DECLARE @C_MATEPRIMA_DTENTPROD    DATETIME;
   DECLARE @C_MATEPRIMA_QTESTOQUE    DOUBLE PRECISION;
   DECLARE @C_MATEPRIMA_QTESTOQMIN   DOUBLE PRECISION;
   DECLARE @C_MATEPRIMA_QTMATEPRIMA  DOUBLE PRECISION;
   DECLARE @V_NRORDENAOP             VARCHAR(10);

   DECLARE @C_REG_NRPEDIDO VARCHAR(10);
   DECLARE @C_REG_NRITEMPED VARCHAR(10);

   DECLARE @C_REG_CDFILSOLI  VARCHAR(4);
   DECLARE @C_REG_CDFILLAN   VARCHAR(4);
   DECLARE @C_REG_NRSOLICMP  VARCHAR(10);
   DECLARE @C_REG_CDPRODUTO  VARCHAR(10);
   DECLARE @C_REG_DTPROGENTR DATETIME;


BEGIN
   SET @V_QTCOMPRAS = 0;
   SET @V_NRORDENAOP = '0000000000';

   IF CHARINDEX('L', @P_IDMRPPEDPREV)  = 0 BEGIN

      /* Verifica produtos pedidos e previstos para venda */
      DECLARE C_PRODPED CURSOR FOR
         SELECT
            IT.CDPRODUTO,
            IT.IDPRODPROD,
            DT.DTMINENTPED - CASE ISNULL(IDAPLICHRSEG, 'L') WHEN 'P' THEN ( PP.QTHRSEGPROD/24) ELSE 0 END  AS DTMINENTPED,
            CASE ISNULL(@P_IDESTOQUE,'N') WHEN 'S' THEN ISNULL(P.QTESTDIACON, 0) ELSE 0 END AS QTDEESTO,
            ISNULL(IT.QTVENDAPCP, 0) AS QTVENDAPCP,
            ISNULL(PF.QTESTOSEGU, 0) AS QTESTOQMIN,
            ISNULL(PROCE.QTPADPROCFAB, 0) AS QTPADPROCFAB,
            ISNULL(PROCE.QTLOTEFABDIA, 0) AS QTLOTEFABDIA,
            ISNULL(OP.QTPRODUCAO, 0) AS QTPRODUCAO,
            ISNULL(OP.QTANDPROD, 0) AS QTANDPROD, 
            ISNULL(OP.QTTOTALPROD, 0) AS QTTOTALPROD
         FROM
            PROCFABRICA PROCE,
            (SELECT
                A.CDPRODUTO,
                CONVERT(DATETIME, CONVERT(VARCHAR, A.DTENTITPED, 103), 103)  AS DTENTITPED,
                MIN(A.DTMINENTPED) AS DTMINENTPED
             FROM
                (SELECT
                    PROD.CDPRODESTO AS CDPRODUTO,
                    CONVERT(DATETIME, CONVERT(VARCHAR, IT.DTENTITPED, 103), 103)  AS DTENTITPED,
                    MIN(IT.DTENTITPED) AS DTMINENTPED
                 FROM
                    PEDIDO P,
                    ITEMPEDIDO IT, PRODUTO PROD,
                    LISTA L
                 WHERE
                    (P.CDFILIAL      = @P_CDFILIAL)                    AND
                    ((P.IDSITPEDIDO  = '2')OR(P.IDSITPEDIDO = '6'  ))  AND
                    ((IT.IDSITITEMPED ='3')OR(IT.IDSITITEMPED ='7'))   AND
                    (IT.CDFILIAL     = P.CDFILIAL)                     AND
                    (IT.NRPEDIDO     = P.NRPEDIDO)                     AND
                    (L.NRLISTA       = @P_STLISTPEDIDO)                AND
                    (IT.NRPEDIDO+'/'+IT.NRITEMPED = L.CDLISTA)         AND
                    (@P_STTODOPEDIDO <> 'T')                           AND
                    (PROD.CDARVPROD BETWEEN @P_CDPRODINI AND @P_CDPRODFIN) AND
                    (CONVERT(VARCHAR, IT.DTENTITPED, 103) BETWEEN @P_DTINI AND @P_DTFIN) AND
                    (IT.CDPRODUTO    = PROD.CDPRODUTO)AND
                     ((CHARINDEX( P.CDDEPTO, @P_CDDEPTO)>0)OR(@P_CDDEPTO IS NULL))AND
                     ( ((@P_ORCDE IS NULL)AND(@P_ORCATE IS NULL)AND(@P_LORC IS NULL))OR
                       (CHARINDEX( P.NRORCAEVENTO, @P_LORC)>0)OR (P.NRORCAEVENTO BETWEEN @P_ORCDE AND @P_ORCATE))AND
                    (CONVERT(DATETIME, CONVERT(VARCHAR, IT.DTENTITPED, 103), 103) BETWEEN @P_DTINI AND @P_DTFIN) AND
                    ((CHARINDEX('P', @P_IDMRPPEDPREV) > 0)OR (CHARINDEX('O', @P_IDMRPPEDPREV) > 0)) AND
                    (NOT EXISTS( SELECT CDFILIAL /* ALTERADO*/
                                  FROM PEDIDOP
                                  WHERE
                                     (CDFILIAL  = IT.CDFILIAL)AND
                                     (NRPEDIDO  = IT.NRPEDIDO)AND
                                     (NRITEMPED = IT.NRITEMPED)))
                 GROUP BY
                    PROD.CDPRODESTO,
                    CONVERT(DATETIME, CONVERT(VARCHAR, IT.DTENTITPED, 103), 103)
                 UNION

                 SELECT
                    PROD.CDPRODESTO AS CDPRODUTO,
                    CONVERT(DATETIME, CONVERT(VARCHAR, IT.DTENTITPED, 103), 103)  AS DTENTITPED,
                    MIN(IT.DTENTITPED) AS DTMINENTPED
                 FROM
                    PEDIDO P,
                    ITEMPEDIDO IT, PRODUTO PROD
                 WHERE
                    (P.CDFILIAL      = @P_CDFILIAL)                    AND
                    ((P.IDSITPEDIDO  = '2')OR(P.IDSITPEDIDO = '6'  ))  AND
                    ((IT.IDSITITEMPED ='3')OR(IT.IDSITITEMPED ='7'))   AND
                    (IT.CDFILIAL     = P.CDFILIAL)                     AND
                    (IT.NRPEDIDO     = P.NRPEDIDO)                     AND
                    (@P_STTODOPEDIDO = 'T')                            AND
                    (PROD.CDARVPROD BETWEEN @P_CDPRODINI AND @P_CDPRODFIN) AND
                    (CONVERT(VARCHAR, IT.DTENTITPED, 103) BETWEEN @P_DTINI AND @P_DTFIN) AND
                    (IT.CDPRODUTO    = PROD.CDPRODUTO)AND
                     ((CHARINDEX( P.CDDEPTO, @P_CDDEPTO)>0)OR(@P_CDDEPTO IS NULL))AND
                     ( ((@P_ORCDE IS NULL)AND(@P_ORCATE IS NULL)AND(@P_LORC IS NULL))OR
                       (CHARINDEX( P.NRORCAEVENTO, @P_LORC)>0)OR (P.NRORCAEVENTO BETWEEN @P_ORCDE AND @P_ORCATE))AND
                    (CONVERT(DATETIME, CONVERT(VARCHAR, IT.DTENTITPED, 103), 103) BETWEEN @P_DTINI AND @P_DTFIN) AND
                    ((CHARINDEX('P', @P_IDMRPPEDPREV) > 0)OR (CHARINDEX('O', @P_IDMRPPEDPREV) > 0)) AND
                    (NOT EXISTS( SELECT CDFILIAL /* ALTERADO*/
                                  FROM PEDIDOP
                                  WHERE
                                     (CDFILIAL  = IT.CDFILIAL)AND
                                     (NRPEDIDO  = IT.NRPEDIDO)AND
                                     (NRITEMPED = IT.NRITEMPED)))
                 GROUP BY
                    PROD.CDPRODESTO,
                    CONVERT(DATETIME, CONVERT(VARCHAR, IT.DTENTITPED, 103), 103)
                 UNION

                 SELECT DISTINCT
                    PROD.CDPRODESTO AS CDPRODUTO,
                    P.DTPREVIVENDA AS DTENTITPED,
                    CONVERT(DATETIME, CONVERT(VARCHAR, P.DTPREVIVENDA), 103) +
                    (CONVERT(DOUBLE PRECISION,SUBSTRING(ISNULL(PA.HRENTRPROD,'0000'),3,2)) + CONVERT(DOUBLE PRECISION,SUBSTRING(ISNULL(PA.HRENTRPROD,'0000'),1,2)))/24
                 FROM
                    PRODUTO PROD,
                    PREVIVENDA P LEFT JOIN PARAMPRODMRP PA ON (P.CDFILIAL  = PA.CDFILIAL)  AND
                                                              (P.CDPRODUTO = PA.CDPRODUTO)
                 WHERE
                    (P.CDFILIAL  = @P_CDFILIAL)                            AND
                    (CONVERT(DATETIME, CONVERT(VARCHAR, P.DTPREVIVENDA, 103), 103) BETWEEN @P_DTINI AND @P_DTFIN) AND
                    (CHARINDEX('V', @P_IDMRPPEDPREV) > 0)AND
                    (PROD.CDPRODUTO = P.CDPRODUTO)

                 UNION

                 SELECT DISTINCT
                    PROD.CDPRODESTO AS CDPRODUTO,
                    D.DTENTRDEMA AS DTENTITPED,
                    CONVERT(DATETIME, CONVERT(VARCHAR, D.DTENTRDEMA), 103) +
                    (CONVERT(DOUBLE PRECISION,SUBSTRING(ISNULL(PA.HRENTRPROD,'0000'),3,2)) + CONVERT(DOUBLE PRECISION,SUBSTRING(ISNULL(PA.HRENTRPROD,'0000'),1,2)))/24
                 FROM
                    PRODUTO PROD,
                    DEMAEXTRAMRP D LEFT JOIN PARAMPRODMRP PA ON (D.CDFILIAL  = PA.CDFILIAL)  AND
                                                                (D.CDPRODUTO = PA.CDPRODUTO)
                 WHERE
                    (D.CDFILIAL    = @P_CDFILIAL) AND
                    (PROD.CDPRODUTO = D.CDPRODUTO)AND
                    (@P_IDCONSDEMA = 'S')

                 UNION

                 SELECT
                     I.CDPRODUTO,
                     I.DTPROGENTR AS DTENTITPED,
                     CONVERT(DATETIME, CONVERT(VARCHAR, I.DTPROGENTR), 103) +
                    (CONVERT(DOUBLE PRECISION,SUBSTRING(ISNULL(PA.HRENTRPROD,'0000'),3,2)) + CONVERT(DOUBLE PRECISION,SUBSTRING(ISNULL(PA.HRENTRPROD,'0000'),1,2)))/24

                  FROM SOLICITA SO, ITEMSOLI I, PRODUTO P, LISTA L, PARAMPRODMRP PA,
                          (SELECT CDFILIAL
                             FROM FILIAL
                            WHERE
                               (( CHARINDEX(CDFILIAL, @P_STLISTAFILI)>0)OR (@P_STTFILISOLI = 'T'))) F
                  WHERE
                        (SO.IDBAIXASOL IS NULL)
                    AND (@P_STTSOLICITA IS NULL)
                    AND (SO.CDFILLANCA = I.CDFILLAN)
                    AND (SO.CDFILSOLI  = I.CDFILSOLI)
                    AND (SO.NRSOLICMP  = I.NRSOLICMP)
                    AND (CDFILIALPED IS NULL)
                    AND (NRPEDIDO    IS NULL)
                    AND  I.CDPRODUTO = P.CDPRODUTO
                    AND (P.CDARVPROD BETWEEN @P_CDPRODINI AND @P_CDPRODFIN )
                    AND (IDSOLIAPROV = 'S')
                    AND DTPROGENTR BETWEEN @P_DTINI AND @P_DTFIN
                    AND I.CDFILLAN+I.CDFILSOLI+I.NRSOLICMP = L.CDLISTA
                    AND (L.NRLISTA   = @P_STLISTASOLI)
                    AND I.CDFILSOLI = F.CDFILIAL
                    AND (@P_CDFILIAL   = PA.CDFILIAL)
                    AND (I.CDPRODUTO  = PA.CDPRODUTO)
                    AND (CHARINDEX( 'S', @P_IDMRPPEDPREV) > 0)
                    AND ( NOT EXISTS( SELECT NRSOLICMP
                                           FROM SOLICITAOP
                                          WHERE
                                             CDFILSOLI = I.CDFILSOLI AND
                                             CDFILLAN  = I.CDFILLAN AND
                                             NRSOLICMP = I.NRSOLICMP AND
                                             CDPRODUTO = I.CDPRODUTO AND
                                             DTPROGENTR = I.DTPROGENTR))

                  UNION

                  SELECT
                     I.CDPRODUTO,
                     I.DTPROGENTR AS DTENTITPED,
                     CONVERT(DATETIME, CONVERT(VARCHAR, I.DTPROGENTR), 103) +
                    (CONVERT(DOUBLE PRECISION,SUBSTRING(ISNULL(PA.HRENTRPROD,'0000'),3,2)) + CONVERT(DOUBLE PRECISION,SUBSTRING(ISNULL(PA.HRENTRPROD,'0000'),1,2)))/24
                  FROM SOLICITA SO, ITEMSOLI I, PRODUTO P, PARAMPRODMRP PA,
                          (SELECT CDFILIAL
                             FROM FILIAL
                            WHERE
                               ((CHARINDEX( CDFILIAL, @P_STLISTAFILI)>0)OR (@P_STTFILISOLI = 'T'))) F
                  WHERE
                        (SO.IDBAIXASOL IS NULL)
                    AND (@P_STTSOLICITA = 'T')
                    AND (SO.CDFILLANCA = I.CDFILLAN)
                    AND (SO.CDFILSOLI  = I.CDFILSOLI)
                    AND (SO.NRSOLICMP  = I.NRSOLICMP)
                    AND (CDFILIALPED IS NULL)
                    AND (NRPEDIDO    IS NULL)
                    AND  I.CDPRODUTO = P.CDPRODUTO
                    AND (P.CDARVPROD BETWEEN @P_CDPRODINI AND @P_CDPRODFIN )
                    AND (IDSOLIAPROV = 'S')
                    AND DTPROGENTR BETWEEN @P_DTINI AND @P_DTFIN
                    AND I.CDFILSOLI = F.CDFILIAL
                    AND (@P_CDFILIAL   = PA.CDFILIAL)
                    AND (I.CDPRODUTO  = PA.CDPRODUTO)
                    AND (CHARINDEX( 'S', @P_IDMRPPEDPREV) > 0)
                    AND ( NOT EXISTS( SELECT NRSOLICMP
                                           FROM SOLICITAOP
                                          WHERE
                                             CDFILSOLI = I.CDFILSOLI AND
                                             CDFILLAN  = I.CDFILLAN AND
                                             NRSOLICMP = I.NRSOLICMP AND
                                             CDPRODUTO = I.CDPRODUTO AND
                                             DTPROGENTR = I.DTPROGENTR))
                ) A
             GROUP BY
                A.CDPRODUTO,
                CONVERT(DATETIME, CONVERT(VARCHAR, A.DTENTITPED, 103), 103)
            ) DT,
            (SELECT
                VEN.CDFILIAL,
                VEN.CDPRODUTO,
                VEN.IDPRODPROD,
                VEN.DTENTITPED,
                ISNULL(SUM(VEN.QTVENDAPCP), 0) AS QTVENDAPCP
             FROM
                (SELECT
                    PED.CDFILIAL,
                    EST.CDPRODUTO,
                    EST.IDPRODPROD,
                    CONVERT(DATETIME, CONVERT(VARCHAR, ITEM.DTENTITPED, 103), 103) AS DTENTITPED,
                    ISNULL(SUM(ITEM.QTITEMPED - ITEM.QTEXPITEMPED)* ISNULL(PROD.VRFATOCONV,1) - SUM(ISNULL( QTRESPRODPED,0)) , 0) AS QTVENDAPCP,
                    'P' AS IDORIGEM
                 FROM
                    PRODUTO PROD,
                    PRODUTO EST,
                    PEDIDO PED,
                    ITEMPEDIDO ITEM LEFT JOIN (SELECT  RESERPRODPED.CDFILIAL, NRPEDIDO, NRITEMPED, QTRESPRODPED
                                               FROM
                                                 RESERPRODPED, PARAMPEDIDO
                                               WHERE
                                                  NRLOTEESTQ IS NOT NULL AND
                                                  PARAMPEDIDO.CDFILIAL = @P_CDFILIAL AND
                                                  IDUTRESELOTE = 'S') R ON (   ITEM.CDFILIAL = R.CDFILIAL AND
                                                                               ITEM.NRPEDIDO = R.NRPEDIDO AND
                                                                               ITEM.NRITEMPED = R.NRITEMPED),
                    LISTA L
                 WHERE
                    (PROD.CDARVPROD BETWEEN @P_CDPRODINI AND @P_CDPRODFIN)    AND
                    (PROD.CDPRODESTO =  EST.CDPRODUTO ) AND
                    (PED.CDFILIAL      = @P_CDFILIAL)                         AND
                    ((PED.IDSITPEDIDO  = '2')OR(PED.IDSITPEDIDO = '6'  ))     AND
                    ((ITEM.IDSITITEMPED ='3')OR(ITEM.IDSITITEMPED ='7'))      AND
                    (ITEM.CDFILIAL     = PED.CDFILIAL)                        AND
                    (ITEM.NRPEDIDO     = PED.NRPEDIDO)                        AND
                    (L.NRLISTA       = @P_STLISTPEDIDO)                       AND
                    (ITEM.NRPEDIDO+'/'+ITEM.NRITEMPED = L.CDLISTA)                AND
                    (@P_STTODOPEDIDO <> 'T') AND
                     ((CHARINDEX( PED.CDDEPTO, @P_CDDEPTO)>0)OR(@P_CDDEPTO IS NULL))AND
                     ( ((@P_ORCDE IS NULL)AND(@P_ORCATE IS NULL)AND(@P_LORC IS NULL))OR
                       (CHARINDEX( PED.NRORCAEVENTO, @P_LORC)>0)OR (PED.NRORCAEVENTO BETWEEN @P_ORCDE AND @P_ORCATE))AND
                    (ITEM.CDPRODUTO    = PROD.CDPRODUTO)                      AND
                    (CONVERT(DATETIME, CONVERT(VARCHAR, ITEM.DTENTITPED, 103), 103) BETWEEN  @P_DTINI AND @P_DTFIN) AND
                    ((CHARINDEX('P', @P_IDMRPPEDPREV) > 0)OR (CHARINDEX('O', @P_IDMRPPEDPREV) > 0)) AND
                    (NOT EXISTS( SELECT CDFILIAL
                                  FROM PEDIDOP
                                  WHERE
                                     (CDFILIAL  = ITEM.CDFILIAL)AND
                                     (NRPEDIDO  = ITEM.NRPEDIDO)AND
                                     (NRITEMPED = ITEM.NRITEMPED)))
                 GROUP BY
                    PED.CDFILIAL,
                    EST.CDPRODUTO,
                    EST.IDPRODPROD,
                    CONVERT(DATETIME, CONVERT(VARCHAR, ITEM.DTENTITPED, 103), 103),
                    PROD.VRFATOCONV

                 UNION

                 SELECT
                    PED.CDFILIAL,
                    EST.CDPRODUTO,
                    EST.IDPRODPROD,
                    CONVERT(DATETIME, CONVERT(VARCHAR, ITEM.DTENTITPED, 103), 103) AS DTENTITPED,
                    ISNULL(SUM(ITEM.QTITEMPED - ITEM.QTEXPITEMPED)* ISNULL(PROD.VRFATOCONV,1) - SUM(ISNULL( QTRESPRODPED,0)) , 0) AS QTVENDAPCP,
                    'P' AS IDORIGEM
                 FROM
                    PRODUTO PROD,
                    PRODUTO EST,
                    PEDIDO PED,
                    ITEMPEDIDO ITEM LEFT JOIN (SELECT  RESERPRODPED.CDFILIAL, NRPEDIDO, NRITEMPED, QTRESPRODPED
                                               FROM
                                                 RESERPRODPED, PARAMPEDIDO
                                               WHERE
                                                  NRLOTEESTQ IS NOT NULL AND
                                                  PARAMPEDIDO.CDFILIAL = @P_CDFILIAL AND
                                                  IDUTRESELOTE = 'S') R ON (   ITEM.CDFILIAL = R.CDFILIAL AND
                                                                               ITEM.NRPEDIDO = R.NRPEDIDO AND
                                                                               ITEM.NRITEMPED = R.NRITEMPED)
                 WHERE
                    (PROD.CDARVPROD BETWEEN @P_CDPRODINI AND @P_CDPRODFIN)    AND
                    (PROD.CDPRODESTO =  EST.CDPRODUTO ) AND
                    (PED.CDFILIAL      = @P_CDFILIAL)                         AND
                    ((PED.IDSITPEDIDO  = '2')OR(PED.IDSITPEDIDO = '6'  ))     AND
                    ((ITEM.IDSITITEMPED ='3')OR(ITEM.IDSITITEMPED ='7'))      AND
                    (ITEM.CDFILIAL     = PED.CDFILIAL)                        AND
                    (ITEM.NRPEDIDO     = PED.NRPEDIDO)                        AND
                    (@P_STTODOPEDIDO = 'T') AND
                     ((CHARINDEX( PED.CDDEPTO, @P_CDDEPTO)>0)OR(@P_CDDEPTO IS NULL))AND
                     ( ((@P_ORCDE IS NULL)AND(@P_ORCATE IS NULL)AND(@P_LORC IS NULL))OR
                       (CHARINDEX( PED.NRORCAEVENTO, @P_LORC)>0)OR (PED.NRORCAEVENTO BETWEEN @P_ORCDE AND @P_ORCATE))AND
                    (ITEM.CDPRODUTO    = PROD.CDPRODUTO)                      AND
                    (CONVERT(DATETIME, CONVERT(VARCHAR, ITEM.DTENTITPED, 103), 103) BETWEEN  @P_DTINI AND @P_DTFIN) AND
                    ((CHARINDEX('P', @P_IDMRPPEDPREV) > 0)OR (CHARINDEX('O', @P_IDMRPPEDPREV) > 0)) AND
                    (NOT EXISTS( SELECT CDFILIAL
                                  FROM PEDIDOP
                                  WHERE
                                     (CDFILIAL  = ITEM.CDFILIAL)AND
                                     (NRPEDIDO  = ITEM.NRPEDIDO)AND
                                     (NRITEMPED = ITEM.NRITEMPED)))
                 GROUP BY
                    PED.CDFILIAL,
                    EST.CDPRODUTO,
                    EST.IDPRODPROD,
                    CONVERT(DATETIME, CONVERT(VARCHAR, ITEM.DTENTITPED, 103), 103),
                    PROD.VRFATOCONV

                 UNION

                 SELECT
                    P.CDFILIAL,
                    EST.CDPRODUTO,
                    EST.IDPRODPROD,
                    P.DTPREVIVENDA AS DTENTITPED,
                    ISNULL(SUM(P.QTPREVIVENDA)* ISNULL(PROD.VRFATOCONV,1), 0) QTVENDAPCP,
                    'V' AS IDORIGEM
                 FROM
                    PRODUTO PROD,
                    PREVIVENDA P, PRODUTO EST
                 WHERE
                    (PROD.CDARVPROD BETWEEN @P_CDPRODINI AND @P_CDPRODFIN)  AND
                    (PROD.CDPRODESTO =  EST.CDPRODUTO ) AND
                    (P.CDFILIAL  = @P_CDFILIAL)                             AND
                    (CONVERT(DATETIME, CONVERT(VARCHAR, P.DTPREVIVENDA, 103), 103) BETWEEN @P_DTINI AND @P_DTFIN) AND
                    (P.CDPRODUTO = PROD.CDPRODUTO)                        AND
                    (CHARINDEX('V', @P_IDMRPPEDPREV) > 0)
                 GROUP BY
                    P.CDFILIAL,
                    EST.CDPRODUTO,
                    EST.IDPRODPROD,
                    P.DTPREVIVENDA,
                    PROD.VRFATOCONV

                 UNION

                 SELECT
                    D.CDFILIAL,
                    EST.CDPRODUTO,
                    EST.IDPRODPROD,
                    D.DTENTRDEMA AS DTENTITPED,
                    ISNULL(SUM(D.QTDEMAEXTRA)* ISNULL(PROD.VRFATOCONV,1), 0) QTVENDAPCP,
                    'D' AS IDORIGEM
                 FROM
                    PRODUTO PROD,
                    DEMAEXTRAMRP D, PRODUTO EST
                 WHERE
                    (D.CDFILIAL   = @P_CDFILIAL)     AND
                    (D.CDPRODUTO  = PROD.CDPRODUTO)  AND
                    (PROD.CDPRODESTO =  EST.CDPRODUTO ) AND
                    (@P_IDCONSDEMA = 'S')
                 GROUP BY
                    D.CDFILIAL,
                    EST.CDPRODUTO,
                    EST.IDPRODPROD,
                    D.DTENTRDEMA,
                    PROD.VRFATOCONV

                 UNION

                 SELECT
                    @P_CDFILIAL,
                    I.CDPRODUTO,
                    P.IDPRODPROD,
                    I.DTPROGENTR AS DTENTITPED,
                    ISNULL(SUM(QTSOLIPROD),0) QTVENDAPCP,
                    'S' AS IDORIGEM
                 FROM SOLICITA SO, ITEMSOLI I, PRODUTO P, LISTA L, PARAMPRODMRP PA,
                         (SELECT CDFILIAL
                            FROM FILIAL
                           WHERE
                              ((CHARINDEX(CDFILIAL, @P_STLISTAFILI)>0)OR (@P_STTFILISOLI = 'T'))) F
                 WHERE
                       (SO.IDBAIXASOL IS NULL)
                   AND (@P_STTSOLICITA IS NULL)
                   AND (SO.CDFILLANCA = I.CDFILLAN)
                   AND (SO.CDFILSOLI  = I.CDFILSOLI)
                   AND (SO.NRSOLICMP  = I.NRSOLICMP)
                   AND (CDFILIALPED IS NULL)
                   AND (NRPEDIDO    IS NULL)
                   AND  I.CDPRODUTO = P.CDPRODUTO
                   AND (P.CDARVPROD BETWEEN @P_CDPRODINI AND @P_CDPRODFIN )
                   AND (IDSOLIAPROV = 'S')
                   AND DTPROGENTR BETWEEN @P_DTINI AND @P_DTFIN
                   AND I.CDFILLAN+I.CDFILSOLI+I.NRSOLICMP = L.CDLISTA
                   AND (L.NRLISTA   = @P_STLISTASOLI)
                   AND I.CDFILSOLI = F.CDFILIAL
                   AND (@P_CDFILIAL   = PA.CDFILIAL)
                   AND (I.CDPRODUTO  = PA.CDPRODUTO)
                   AND (CHARINDEX('S', @P_IDMRPPEDPREV) > 0)
                   AND ( NOT EXISTS( SELECT NRSOLICMP
                                          FROM SOLICITAOP
                                         WHERE
                                            CDFILSOLI = I.CDFILSOLI AND
                                            CDFILLAN  = I.CDFILLAN AND
                                            NRSOLICMP = I.NRSOLICMP AND
                                            CDPRODUTO = I.CDPRODUTO AND
                                            DTPROGENTR = I.DTPROGENTR))
                 GROUP BY
                    I.CDPRODUTO,
                    P.IDPRODPROD,
                    I.DTPROGENTR

                 UNION

                 SELECT
                    @P_CDFILIAL,
                    I.CDPRODUTO,
                    P.IDPRODPROD,
                    I.DTPROGENTR AS DTENTITPED,
                    ISNULL(SUM(QTSOLIPROD),0) QTVENDAPCP,
                    'S' AS IDORIGEM
                 FROM SOLICITA SO, ITEMSOLI I, PRODUTO P, PARAMPRODMRP PA,
                         (SELECT CDFILIAL
                            FROM FILIAL
                           WHERE
                              ((CHARINDEX( CDFILIAL, @P_STLISTAFILI)>0)OR ( @P_STTFILISOLI = 'T'))) F
                 WHERE
                       (SO.IDBAIXASOL IS NULL)
                   AND (@P_STTSOLICITA = 'T')
                   AND (SO.CDFILLANCA = I.CDFILLAN)
                   AND (SO.CDFILSOLI  = I.CDFILSOLI)
                   AND (SO.NRSOLICMP  = I.NRSOLICMP)
                   AND (CDFILIALPED IS NULL)
                   AND (NRPEDIDO    IS NULL)
                   AND  I.CDPRODUTO = P.CDPRODUTO
                   AND (P.CDARVPROD BETWEEN @P_CDPRODINI AND @P_CDPRODFIN )
                   AND (IDSOLIAPROV = 'S')
                   AND DTPROGENTR BETWEEN @P_DTINI AND @P_DTFIN
                   AND I.CDFILSOLI = F.CDFILIAL
                   AND (@P_CDFILIAL   = PA.CDFILIAL)
                   AND (I.CDPRODUTO  = PA.CDPRODUTO)
                   AND (CHARINDEX( 'S', @P_IDMRPPEDPREV) > 0)
                   AND ( NOT EXISTS( SELECT NRSOLICMP
                                          FROM SOLICITAOP
                                         WHERE
                                            CDFILSOLI = I.CDFILSOLI AND
                                            CDFILLAN  = I.CDFILLAN AND
                                            NRSOLICMP = I.NRSOLICMP AND
                                            CDPRODUTO = I.CDPRODUTO AND
                                            DTPROGENTR = I.DTPROGENTR))
                 GROUP BY
                    I.CDPRODUTO,
                    P.IDPRODPROD,
                    I.DTPROGENTR
                ) VEN
             GROUP BY
                VEN.CDFILIAL,
                VEN.CDPRODUTO,
                VEN.IDPRODPROD,
                VEN.DTENTITPED
            ) IT LEFT JOIN
                 (SELECT
                     A.CDFILIAL,
                     A.CDPRODUTO, 
                     A.IDGERAOPCONF,
                     SUM(A.QTPRODUCAO) AS QTPRODUCAO,
                     SUM(A.QTANDPROD) AS QTANDPROD,
                     ISNULL(SUM(A.QTTOTALPROD), 0) AS QTTOTALPROD
                  FROM
                     (SELECT
                         L.CDFILIAL,
                         L.CDPRODUTO, P.IDGERAOPCONF,
                         0 AS QTPRODUCAO,
                         ISNULL(SUM(L.QTPRODLOTE), 0) AS QTANDPROD,
                         ISNULL(SUM(L.QTPRODLOTE), 0) AS QTTOTALPROD
                      FROM
                         LOTE L, PARAMPRODMRP P
                      WHERE
                         (L.CDFILIAL     = @P_CDFILIAL)  AND
                         (L.IDSTPRODLOTE = '3')AND
                         (P.CDFILIAL = L.CDFILIAL) AND
                         (P.CDPRODUTO = L.CDPRODUTO) AND
                         (P.IDGERAOPCONF = 'S')
                      GROUP BY
                         L.CDFILIAL,
                         L.CDPRODUTO, P.IDGERAOPCONF

                      UNION ALL

                      SELECT
                         O.CDFILIAL,
                         O.CDPRODUTO,
                         P.IDGERAOPCONF,
                         ISNULL(SUM(QTAPRODPREOP),0) AS QTPRODUCAO,
                         0 AS QTANDPROD,
                         ISNULL(SUM(QTAPRODPREOP), 0) AS QTTOTALPROD
                      FROM
                         ORDPRODUCAO O, PARAMPRODMRP P
                      WHERE
                         (O.CDFILIAL     = @P_CDFILIAL)  AND
                         (O.IDSITORDPROD = '1')          AND
                         (O.IDCONFIRMAOP = 'S')          AND
                         (P.CDFILIAL = O.CDFILIAL) AND
                         (P.CDPRODUTO = O.CDPRODUTO) AND
                         (P.IDGERAOPCONF = 'S')
                      GROUP BY
                         O.CDFILIAL,
                         O.CDPRODUTO, P.IDGERAOPCONF
                     ) A
                  GROUP BY
                     A.CDFILIAL,
                     A.CDPRODUTO, A.IDGERAOPCONF
                 ) OP  ON (IT.CDFILIAL  = OP.CDFILIAL)  AND
                          (IT.CDPRODUTO = OP.CDPRODUTO)
                 LEFT JOIN  POSESTCONS P  ON (IT.CDFILIAL  = P.CDFILIAL)   AND
                                             (IT.CDPRODUTO = P.CDPRODUTO)
                 LEFT JOIN ESTOSEGU PF ON (IT.CDFILIAL  = PF.CDFILIAL)  AND
                                          (IT.CDPRODUTO = PF.CDPRODESTO)
                 LEFT JOIN PARAMPRODMRP PP ON ( IT.CDFILIAL    = PP.CDFILIAL AND
                                                IT.CDPRODUTO   = PP.CDPRODUTO)
         WHERE
            (PROCE.CDFILIAL     = IT.CDFILIAL)                       AND
            (PROCE.CDPRODUTO    = IT.CDPRODUTO)                      AND
            (PROCE.IDPROFABPADR = 'S')                               AND
            ((P.DTPOSESTCON = (SELECT MAX(PO.DTPOSESTCON)
                               FROM POSESTCONS PO
                               WHERE PO.CDFILIAL    = IT.CDFILIAL  AND
                                     PO.CDPRODUTO   = IT.CDPRODUTO AND
                                     PO.DTPOSESTCON <= (GETDATE() + 1)))    OR
              P.DTPOSESTCON IS NULL)                                    AND
            (IT.CDPRODUTO  = DT.CDPRODUTO)                              AND
            (IT.DTENTITPED = DT.DTENTITPED)
         ORDER BY
            DT.DTMINENTPED,
            IT.CDPRODUTO

      OPEN C_PRODPED;
      FETCH NEXT FROM C_PRODPED INTO @C_PRODPED_CDPRODUTO, @C_PRODPED_IDPRODPROD, @C_PRODPED_DTMINENTPED,
                                     @C_PRODPED_QTDEESTO, @C_PRODPED_QTVENDAPCP, @C_PRODPED_QTESTOQMIN,
                                     @C_PRODPED_QTPADPROCFAB, @C_PRODPED_QTLOTEFABDIA,
                                     @C_PRODPED_QTPRODUCAO, @C_PRODPED_QTANDPROD, @C_PRODPED_QTTOTALPROD;


      WHILE (@@FETCH_STATUS = 0) BEGIN

         SELECT @V_NRORDENAOP = RIGHT(REPLICATE('0',10)+RTRIM(CONVERT(CHAR, CONVERT(NUMERIC, ISNULL(MAX(NRORDENAOP),'0')) + 1)),10)
         FROM CALEPRODMRP
         WHERE CDFILIAL = @P_CDFILIAL
           AND DTPROGPRODCP = @P_DTMRP;

         EXEC GERA_MRP_PRODUTO @C_PRODPED_QTANDPROD, @C_PRODPED_QTTOTALPROD, 'N', @P_CDFILIAL,
                               @P_DTMRP, @C_PRODPED_CDPRODUTO, @C_PRODPED_IDPRODPROD,
                               @C_PRODPED_DTMINENTPED, @C_PRODPED_QTDEESTO, @C_PRODPED_QTESTOQMIN,
                               @C_PRODPED_QTPADPROCFAB, @C_PRODPED_QTLOTEFABDIA, @C_PRODPED_QTVENDAPCP,
                               @C_PRODPED_QTPRODUCAO, @P_DTINI, @P_DTFIN,
                               @P_CDPRODINI, @P_CDPRODFIN, @P_IDMRPPEDPREV, @P_IDESTOQUE, @P_IDCONSDEMA,
                               @V_NRORDENAOP, '0', @C_PRODPED_DTMINENTPED, @C_PRODPED_CDPRODUTO, @V_NRORDENAOP;

         FETCH NEXT FROM C_PRODPED INTO @C_PRODPED_CDPRODUTO, @C_PRODPED_IDPRODPROD, @C_PRODPED_DTMINENTPED,
                                        @C_PRODPED_QTDEESTO, @C_PRODPED_QTVENDAPCP, @C_PRODPED_QTESTOQMIN,
                                        @C_PRODPED_QTPADPROCFAB, @C_PRODPED_QTLOTEFABDIA,
                                        @C_PRODPED_QTPRODUCAO, @C_PRODPED_QTANDPROD, @C_PRODPED_QTTOTALPROD;
      END;
      CLOSE C_PRODPED;
      DEALLOCATE C_PRODPED;
   END
   ELSE BEGIN
      /* Verifica produtos pedidos (Plano de Produo) */
      DECLARE C_PRODPLAN CURSOR FOR
         SELECT
            IT.CDPRODUTO,
            IT.IDPRODPROD,
            DT.DTMINENTPED - CASE ISNULL(IDAPLICHRSEG, 'L') WHEN 'P' THEN ( PP.QTHRSEGPROD/24) ELSE 0 END  AS DTMINENTPED,
            CASE ISNULL( @P_IDESTOQUE,'N') WHEN 'S' THEN ISNULL(P.QTESTDIACON, 0) ELSE 0 END AS QTDEESTO,
            ISNULL(IT.QTVENDAPCP, 0) AS QTVENDAPCP,
            ISNULL(PF.QTESTOSEGU, 0) AS QTESTOQMIN,
            ISNULL(PROCE.QTPADPROCFAB, 0) AS QTPADPROCFAB,
            ISNULL(PROCE.QTLOTEFABDIA, 0) AS QTLOTEFABDIA,
            ISNULL(OP.QTPRODUCAO, 0) AS QTPRODUCAO,
            ISNULL(OP.QTANDPROD, 0) AS QTANDPROD,
            ISNULL(OP.QTTOTALPROD, 0) AS QTTOTALPROD
         FROM
            PROCFABRICA PROCE,
            (SELECT
                A.CDPRODUTO,
                CONVERT(DATETIME, CONVERT(VARCHAR, A.DTENTITPED, 103), 103)  AS DTENTITPED,
                MIN(A.DTMINENTPED) AS DTMINENTPED
             FROM
                (SELECT
                    PROD.CDPRODESTO AS CDPRODUTO,
                    CONVERT(DATETIME, CONVERT(VARCHAR, IT.DTENTITPED, 103), 103) AS DTENTITPED,
                    MIN(IT.DTENTITPED) AS DTMINENTPED
                 FROM
                    ITEMPEDIDO IT, PEDIDO P, PRODUTO PROD, LISTA L
                 WHERE
                    (IT.CDFILIAL = @P_CDFILIAL)AND
                    (P.CDFILIAL = IT.CDFILIAL)AND
                    (P.NRPEDIDO = IT.NRPEDIDO)AND
                    ((CHARINDEX( P.CDDEPTO, @P_CDDEPTO)>0)OR(@P_CDDEPTO IS NULL))AND
                    (((@P_ORCDE IS NULL)AND(@P_ORCATE IS NULL)AND(@P_LORC IS NULL))OR
                      (CHARINDEX( P.NRORCAEVENTO, @P_LORC)>0)OR (P.NRORCAEVENTO BETWEEN @P_ORCDE AND @P_ORCATE))AND
                    (PROD.CDARVPROD BETWEEN @P_CDPRODINI AND @P_CDPRODFIN) AND
                    (L.NRLISTA       = @P_STLISTPEDIDO)                       AND
                    (IT.NRPEDIDO+'/'+IT.NRITEMPED = L.CDLISTA)                AND
                    (@P_STTODOPEDIDO <> 'T')AND
                    (IT.CDPRODUTO    = PROD.CDPRODUTO)AND
                    (P.IDSITPEDIDO = '2'  )         AND
                    (IT.IDSITITEMPED = '3')         AND
                    (CONVERT(VARCHAR, IT.DTENTITPED, 103) BETWEEN @P_DTINI AND @P_DTFIN) AND
                    (NOT EXISTS( SELECT CDFILIAL /* ALTERADO*/
                                  FROM PEDIDOP
                                  WHERE
                                     (CDFILIAL  = IT.CDFILIAL)AND
                                     (NRPEDIDO  = IT.NRPEDIDO)AND
                                     (NRITEMPED = IT.NRITEMPED)))
                 GROUP BY
                    PROD.CDPRODESTO,
                    CONVERT(DATETIME, CONVERT(VARCHAR, IT.DTENTITPED, 103), 103)

                 UNION

                 SELECT
                    PROD.CDPRODESTO AS CDPRODUTO,
                    CONVERT(DATETIME, CONVERT(VARCHAR, IT.DTENTITPED, 103), 103) AS DTENTITPED,
                    MIN(IT.DTENTITPED) AS DTMINENTPED
                 FROM
                    ITEMPEDIDO IT, PEDIDO P, PRODUTO PROD
                 WHERE
                    (IT.CDFILIAL = @P_CDFILIAL)AND
                    (P.CDFILIAL = IT.CDFILIAL)AND
                    (P.NRPEDIDO = IT.NRPEDIDO)AND
                    ((CHARINDEX( P.CDDEPTO, @P_CDDEPTO)>0)OR(@P_CDDEPTO IS NULL))AND
                    (((@P_ORCDE IS NULL)AND(@P_ORCATE IS NULL)AND(@P_LORC IS NULL))OR
                      (CHARINDEX( P.NRORCAEVENTO, @P_LORC)>0)OR (P.NRORCAEVENTO BETWEEN @P_ORCDE AND @P_ORCATE))AND
                    (PROD.CDARVPROD BETWEEN @P_CDPRODINI AND @P_CDPRODFIN) AND
                    (@P_STTODOPEDIDO = 'T')AND
                    (IT.CDPRODUTO    = PROD.CDPRODUTO)AND
                    (P.IDSITPEDIDO = '2'  )         AND
                    (IT.IDSITITEMPED = '3')         AND
                    (CONVERT(VARCHAR, IT.DTENTITPED, 103) BETWEEN @P_DTINI AND @P_DTFIN) AND
                    (NOT EXISTS( SELECT CDFILIAL /* ALTERADO*/
                                  FROM PEDIDOP
                                  WHERE
                                     (CDFILIAL  = IT.CDFILIAL)AND
                                     (NRPEDIDO  = IT.NRPEDIDO)AND
                                     (NRITEMPED = IT.NRITEMPED)))
                 GROUP BY
                    PROD.CDPRODESTO,
                    CONVERT(DATETIME, CONVERT(VARCHAR, IT.DTENTITPED, 103), 103)

                 UNION

                 SELECT DISTINCT
                    PROD.CDPRODESTO AS CDPRODUTO,
                    D.DTENTRDEMA AS DTENTITPED,
                    CONVERT(DATETIME, CONVERT(VARCHAR, D.DTENTRDEMA), 103) +
                    (CONVERT(DOUBLE PRECISION,SUBSTRING(ISNULL(PA.HRENTRPROD,'0000'),3,2)) + CONVERT(DOUBLE PRECISION,SUBSTRING(ISNULL(PA.HRENTRPROD,'0000'),1,2)))/24
                 FROM
                    PRODUTO PROD,
                    DEMAEXTRAMRP D LEFT JOIN PARAMPRODMRP PA ON (D.CDFILIAL  = PA.CDFILIAL)  AND
                                                                (D.CDPRODUTO = PA.CDPRODUTO)
                 WHERE
                    (D.CDFILIAL    = @P_CDFILIAL)       AND
                    (@P_IDCONSDEMA = 'S') AND
                    (PROD.CDPRODUTO = D.CDPRODUTO)


                 UNION


                 SELECT
                     I.CDPRODUTO,
                     I.DTPROGENTR AS DTENTITPED,
                     CONVERT(DATETIME, CONVERT(VARCHAR, I.DTPROGENTR), 103) +
                    (CONVERT(DOUBLE PRECISION,SUBSTRING(ISNULL(PA.HRENTRPROD,'0000'),3,2)) + CONVERT(DOUBLE PRECISION,SUBSTRING(ISNULL(PA.HRENTRPROD,'0000'),1,2)))/24

                  FROM SOLICITA SO, ITEMSOLI I, PRODUTO P, LISTA L, PARAMPRODMRP PA,
                          (SELECT CDFILIAL
                             FROM FILIAL
                            WHERE
                               (( CHARINDEX(CDFILIAL, @P_STLISTAFILI)>0)OR (@P_STTFILISOLI = 'T'))) F
                  WHERE
                        (SO.IDBAIXASOL IS NULL)
                    AND (@P_STTSOLICITA IS NULL)
                    AND (SO.CDFILLANCA = I.CDFILLAN)
                    AND (SO.CDFILSOLI  = I.CDFILSOLI)
                    AND (SO.NRSOLICMP  = I.NRSOLICMP)
                    AND (CDFILIALPED IS NULL)
                    AND (NRPEDIDO    IS NULL)
                    AND  I.CDPRODUTO = P.CDPRODUTO
                    AND (P.CDARVPROD BETWEEN @P_CDPRODINI AND @P_CDPRODFIN )
                    AND (IDSOLIAPROV = 'S')
                    AND DTPROGENTR BETWEEN @P_DTINI AND @P_DTFIN
                    AND I.CDFILLAN+I.CDFILSOLI+I.NRSOLICMP = L.CDLISTA
                    AND (L.NRLISTA   = @P_STLISTASOLI)
                    AND I.CDFILSOLI = F.CDFILIAL
                    AND (@P_CDFILIAL   = PA.CDFILIAL)
                    AND (I.CDPRODUTO  = PA.CDPRODUTO)
                    AND (CHARINDEX( 'S', @P_IDMRPPEDPREV) > 0)
                    AND ( NOT EXISTS( SELECT NRSOLICMP
                                           FROM SOLICITAOP
                                          WHERE
                                             CDFILSOLI = I.CDFILSOLI AND
                                             CDFILLAN  = I.CDFILLAN AND
                                             NRSOLICMP = I.NRSOLICMP AND
                                             CDPRODUTO = I.CDPRODUTO AND
                                             DTPROGENTR = I.DTPROGENTR))

                  UNION

                  SELECT
                     I.CDPRODUTO,
                     I.DTPROGENTR AS DTENTITPED,
                     CONVERT(DATETIME, CONVERT(VARCHAR, I.DTPROGENTR), 103) +
                    (CONVERT(DOUBLE PRECISION,SUBSTRING(ISNULL(PA.HRENTRPROD,'0000'),3,2)) + CONVERT(DOUBLE PRECISION,SUBSTRING(ISNULL(PA.HRENTRPROD,'0000'),1,2)))/24
                  FROM SOLICITA SO, ITEMSOLI I, PRODUTO P, PARAMPRODMRP PA,
                          (SELECT CDFILIAL
                             FROM FILIAL
                            WHERE
                               ((CHARINDEX( CDFILIAL, @P_STLISTAFILI)>0)OR (@P_STTFILISOLI = 'T'))) F
                  WHERE
                        (SO.IDBAIXASOL IS NULL)
                    AND (@P_STTSOLICITA = 'T')
                    AND (SO.CDFILLANCA = I.CDFILLAN)
                    AND (SO.CDFILSOLI  = I.CDFILSOLI)
                    AND (SO.NRSOLICMP  = I.NRSOLICMP)
                    AND (CDFILIALPED IS NULL)
                    AND (NRPEDIDO    IS NULL)
                    AND  I.CDPRODUTO = P.CDPRODUTO
                    AND (P.CDARVPROD BETWEEN @P_CDPRODINI AND @P_CDPRODFIN )
                    AND (IDSOLIAPROV = 'S')
                    AND DTPROGENTR BETWEEN @P_DTINI AND @P_DTFIN
                    AND I.CDFILSOLI = F.CDFILIAL
                    AND (@P_CDFILIAL   = PA.CDFILIAL)
                    AND (I.CDPRODUTO  = PA.CDPRODUTO)
                    AND (CHARINDEX( 'S', @P_IDMRPPEDPREV) > 0)
                    AND ( NOT EXISTS( SELECT NRSOLICMP
                                           FROM SOLICITAOP
                                          WHERE
                                             CDFILSOLI = I.CDFILSOLI AND
                                             CDFILLAN  = I.CDFILLAN AND
                                             NRSOLICMP = I.NRSOLICMP AND
                                             CDPRODUTO = I.CDPRODUTO AND
                                             DTPROGENTR = I.DTPROGENTR))
                ) A
             GROUP BY
                A.CDPRODUTO,
                CONVERT(DATETIME, CONVERT(VARCHAR, A.DTENTITPED, 103), 103)
            ) DT,
            (SELECT
                VEN.CDFILIAL,
                VEN.CDPRODUTO,
                PROD.IDPRODPROD,
                VEN.DTENTITPED,
                ISNULL(SUM(VEN.QTVENDAPCP), 0) AS QTVENDAPCP
             FROM
                PRODUTO PROD,
                (SELECT
                    IT.CDFILIAL,
                    PROD.CDPRODESTO AS CDPRODUTO,
                    CONVERT(DATETIME, CONVERT(VARCHAR, IT.DTENTITPED, 103), 103) AS DTENTITPED,
                    ISNULL(SUM(IT.QTITEMPED - IT.QTEXPITEMPED)* ISNULL(PROD.VRFATOCONV,1) - SUM(ISNULL( QTRESPRODPED,0)) , 0) AS QTVENDAPCP,
                    'P' AS IDORIGEM
                 FROM
                    ITEMPEDIDO IT LEFT JOIN (SELECT  RESERPRODPED.CDFILIAL, NRPEDIDO, NRITEMPED, QTRESPRODPED
                                               FROM
                                                 RESERPRODPED, PARAMPEDIDO
                                               WHERE
                                                  NRLOTEESTQ IS NOT NULL AND
                                                  PARAMPEDIDO.CDFILIAL = @P_CDFILIAL AND
                                                  IDUTRESELOTE = 'S') R ON (   IT.CDFILIAL = R.CDFILIAL AND
                                                                               IT.NRPEDIDO = R.NRPEDIDO AND
                                                                               IT.NRITEMPED = R.NRITEMPED),
                    PEDIDO P, PRODUTO PROD, LISTA L
                 WHERE
                   (IT.CDFILIAL  = @P_CDFILIAL)AND
                   (IT.IDSITITEMPED = '3')    AND
                   (CONVERT(VARCHAR, IT.DTENTITPED, 103) BETWEEN @P_DTINI AND @P_DTFIN) AND
                   (PROD.CDARVPROD BETWEEN @P_CDPRODINI AND @P_CDPRODFIN) AND
                   (L.NRLISTA       = @P_STLISTPEDIDO)                       AND
                   (IT.NRPEDIDO+'/'+IT.NRITEMPED = L.CDLISTA)                AND
                   (@P_STTODOPEDIDO <> 'T')AND
                   (P.IDSITPEDIDO = '2')      AND
                   (IT.CDFILIAL = P.CDFILIAL  )AND
                   (IT.NRPEDIDO = P.NRPEDIDO  )AND
                   (IT.CDPRODUTO    = PROD.CDPRODUTO)AND
                   (NOT EXISTS( SELECT CDFILIAL
                                  FROM PEDIDOP
                                  WHERE
                                     (CDFILIAL  = IT.CDFILIAL)AND
                                     (NRPEDIDO  = IT.NRPEDIDO)AND
                                     (NRITEMPED = IT.NRITEMPED)))
                 GROUP BY
                    IT.CDFILIAL,
                    PROD.CDPRODESTO,
                    CONVERT(DATETIME, CONVERT(VARCHAR, IT.DTENTITPED, 103), 103), PROD.VRFATOCONV

                 UNION

                 SELECT
                    IT.CDFILIAL,
                    PROD.CDPRODESTO AS CDPRODUTO,
                    CONVERT(DATETIME, CONVERT(VARCHAR, IT.DTENTITPED, 103), 103) AS DTENTITPED,
                    ISNULL(SUM(IT.QTITEMPED - IT.QTEXPITEMPED)* ISNULL(PROD.VRFATOCONV,1) - SUM(ISNULL( QTRESPRODPED,0)) , 0) AS QTVENDAPCP,
                     'P' AS IDORIGEM
                 FROM
                    ITEMPEDIDO IT LEFT JOIN (SELECT  RESERPRODPED.CDFILIAL, NRPEDIDO, NRITEMPED, QTRESPRODPED
                                               FROM
                                                 RESERPRODPED, PARAMPEDIDO
                                               WHERE
                                                  NRLOTEESTQ IS NOT NULL AND
                                                  PARAMPEDIDO.CDFILIAL = @P_CDFILIAL AND
                                                  IDUTRESELOTE = 'S') R ON (   IT.CDFILIAL = R.CDFILIAL AND
                                                                               IT.NRPEDIDO = R.NRPEDIDO AND
                                                                               IT.NRITEMPED = R.NRITEMPED),
                    PEDIDO P, PRODUTO PROD
                 WHERE
                   (IT.CDFILIAL  = @P_CDFILIAL)AND
                   (IT.IDSITITEMPED = '3')    AND
                   (CONVERT(VARCHAR, IT.DTENTITPED, 103) BETWEEN @P_DTINI AND @P_DTFIN) AND
                   (PROD.CDARVPROD BETWEEN @P_CDPRODINI AND @P_CDPRODFIN) AND
                   (@P_STTODOPEDIDO = 'T')AND
                   (P.IDSITPEDIDO = '2')      AND
                   (IT.CDFILIAL = P.CDFILIAL  )AND
                   (IT.NRPEDIDO = P.NRPEDIDO  )AND
                   (IT.CDPRODUTO    = PROD.CDPRODUTO)AND
                   (NOT EXISTS( SELECT CDFILIAL
                                  FROM PEDIDOP
                                  WHERE
                                     (CDFILIAL  = IT.CDFILIAL)AND
                                     (NRPEDIDO  = IT.NRPEDIDO)AND
                                     (NRITEMPED = IT.NRITEMPED)))
                 GROUP BY
                    IT.CDFILIAL,
                    PROD.CDPRODESTO,
                    CONVERT(DATETIME, CONVERT(VARCHAR, IT.DTENTITPED, 103), 103), PROD.VRFATOCONV

                 UNION

                 SELECT
                    D.CDFILIAL,
                    PROD.CDPRODESTO AS CDPRODUTO,
                    D.DTENTRDEMA AS DTENTITPED,
                    ISNULL(SUM(D.QTDEMAEXTRA)* ISNULL(PROD.VRFATOCONV,1), 0) QTVENDAPCP,
                     'D' AS IDORIGEM
                 FROM
                    DEMAEXTRAMRP D, PRODUTO PROD
                 WHERE
                    (D.CDFILIAL   = @P_CDFILIAL)     AND
                    ( D.CDPRODUTO = PROD.CDPRODUTO)AND
                    (@P_IDCONSDEMA = 'S')
                 GROUP BY
                    D.CDFILIAL,
                    PROD.CDPRODESTO,
                    D.DTENTRDEMA, PROD.VRFATOCONV


                 UNION

                 SELECT
                    @P_CDFILIAL,
                    I.CDPRODUTO,
                    I.DTPROGENTR AS DTENTITPED,
                    ISNULL(SUM(QTSOLIPROD),0) QTVENDAPCP,
                     'S' AS IDORIGEM
                 FROM SOLICITA SO, ITEMSOLI I, PRODUTO P, LISTA L, PARAMPRODMRP PA,
                         (SELECT CDFILIAL
                            FROM FILIAL
                           WHERE
                              ((CHARINDEX(CDFILIAL, @P_STLISTAFILI)>0)OR (@P_STTFILISOLI = 'T'))) F
                 WHERE
                       (SO.IDBAIXASOL IS NULL)
                   AND (@P_STTSOLICITA IS NULL)
                   AND (SO.CDFILLANCA = I.CDFILLAN)
                   AND (SO.CDFILSOLI  = I.CDFILSOLI)
                   AND (SO.NRSOLICMP  = I.NRSOLICMP)
                   AND (CDFILIALPED IS NULL)
                   AND (NRPEDIDO    IS NULL)
                   AND  I.CDPRODUTO = P.CDPRODUTO
                   AND (P.CDARVPROD BETWEEN @P_CDPRODINI AND @P_CDPRODFIN )
                   AND (IDSOLIAPROV = 'S')
                   AND DTPROGENTR BETWEEN @P_DTINI AND @P_DTFIN
                   AND I.CDFILLAN+I.CDFILSOLI+I.NRSOLICMP = L.CDLISTA
                   AND (L.NRLISTA   = @P_STLISTASOLI)
                   AND I.CDFILSOLI = F.CDFILIAL
                   AND (@P_CDFILIAL   = PA.CDFILIAL)
                   AND (I.CDPRODUTO  = PA.CDPRODUTO)
                   AND (CHARINDEX('S', @P_IDMRPPEDPREV) > 0)
                   AND ( NOT EXISTS( SELECT NRSOLICMP
                                          FROM SOLICITAOP
                                         WHERE
                                            CDFILSOLI = I.CDFILSOLI AND
                                            CDFILLAN  = I.CDFILLAN AND
                                            NRSOLICMP = I.NRSOLICMP AND
                                            CDPRODUTO = I.CDPRODUTO AND
                                            DTPROGENTR = I.DTPROGENTR))
                 GROUP BY
                    I.CDPRODUTO,
                    I.DTPROGENTR

                 UNION

                 SELECT
                    @P_CDFILIAL,
                    I.CDPRODUTO,
                    I.DTPROGENTR AS DTENTITPED,
                    ISNULL(SUM(QTSOLIPROD),0) QTVENDAPCP,
                     'S' AS IDORIGEM
                 FROM SOLICITA SO, ITEMSOLI I, PRODUTO P, PARAMPRODMRP PA,
                         (SELECT CDFILIAL
                            FROM FILIAL
                           WHERE
                              ((CHARINDEX( CDFILIAL, @P_STLISTAFILI)>0)OR ( @P_STTFILISOLI = 'T'))) F
                 WHERE
                       (SO.IDBAIXASOL IS NULL)
                   AND (@P_STTSOLICITA = 'T')
                   AND (SO.CDFILLANCA = I.CDFILLAN)
                   AND (SO.CDFILSOLI  = I.CDFILSOLI)
                   AND (SO.NRSOLICMP  = I.NRSOLICMP)
                   AND (CDFILIALPED IS NULL)
                   AND (NRPEDIDO    IS NULL)
                   AND  I.CDPRODUTO = P.CDPRODUTO
                   AND (P.CDARVPROD BETWEEN @P_CDPRODINI AND @P_CDPRODFIN )
                   AND (IDSOLIAPROV = 'S')
                   AND DTPROGENTR BETWEEN @P_DTINI AND @P_DTFIN
                   AND I.CDFILSOLI = F.CDFILIAL
                   AND (@P_CDFILIAL   = PA.CDFILIAL)
                   AND (I.CDPRODUTO  = PA.CDPRODUTO)
                   AND (CHARINDEX( 'S', @P_IDMRPPEDPREV) > 0)
                   AND ( NOT EXISTS( SELECT NRSOLICMP
                                          FROM SOLICITAOP
                                         WHERE
                                            CDFILSOLI = I.CDFILSOLI AND
                                            CDFILLAN  = I.CDFILLAN AND
                                            NRSOLICMP = I.NRSOLICMP AND
                                            CDPRODUTO = I.CDPRODUTO AND
                                            DTPROGENTR = I.DTPROGENTR))
                 GROUP BY
                    I.CDPRODUTO,
                    I.DTPROGENTR
                ) VEN
             WHERE
                (VEN.CDPRODUTO = PROD.CDPRODUTO)
             GROUP BY
                VEN.CDFILIAL,
                VEN.CDPRODUTO,
                PROD.IDPRODPROD,
                VEN.DTENTITPED

            ) IT LEFT JOIN
                 (SELECT
                     A.CDFILIAL,
                     A.CDPRODUTO, A.IDGERAOPCONF,
                     SUM(A.QTPRODUCAO) AS QTPRODUCAO,
                     SUM(A.QTANDPROD) AS QTANDPROD,
                     ISNULL(SUM(A.QTTOTALPROD), 0) AS QTTOTALPROD
                  FROM
                     (SELECT
                         L.CDFILIAL,
                         L.CDPRODUTO, P.IDGERAOPCONF,
                         0 AS QTPRODUCAO,
                         ISNULL(SUM(L.QTPRODLOTE), 0) AS QTANDPROD,
                         ISNULL(SUM(L.QTPRODLOTE), 0) AS QTTOTALPROD
                      FROM
                         LOTE L, PARAMPRODMRP P
                      WHERE
                         (L.CDFILIAL     = @P_CDFILIAL)  AND
                         (L.IDSTPRODLOTE = '3') AND
                         (P.CDFILIAL = L.CDFILIAL) AND
                         (P.CDPRODUTO = L.CDPRODUTO) AND
                         (P.IDGERAOPCONF = 'S')
                      GROUP BY
                         L.CDFILIAL,
                         L.CDPRODUTO, P.IDGERAOPCONF

                      UNION ALL

                      SELECT
                         O.CDFILIAL,
                         O.CDPRODUTO, P.IDGERAOPCONF,
                         ISNULL(SUM(QTAPRODPREOP),0) AS QTPRODUCAO,
                         0 AS QTANDPROD,
                         ISNULL(SUM(QTAPRODPREOP), 0) AS QTTOTALPROD
                      FROM
                         ORDPRODUCAO O, PARAMPRODMRP P
                      WHERE
                         (O.CDFILIAL     = @P_CDFILIAL)  AND
                         (O.IDSITORDPROD = '1')          AND
                         (O.IDCONFIRMAOP = 'S')          AND
                         (P.CDFILIAL = O.CDFILIAL) AND
                         (P.CDPRODUTO = O.CDPRODUTO) AND
                         (P.IDGERAOPCONF = 'S')
                      GROUP BY
                         O.CDFILIAL,
                         O.CDPRODUTO, P.IDGERAOPCONF
                     ) A
                  GROUP BY
                     A.CDFILIAL,
                     A.CDPRODUTO, A.IDGERAOPCONF
                 ) OP  ON (IT.CDFILIAL  = OP.CDFILIAL)  AND
                          (IT.CDPRODUTO = OP.CDPRODUTO)
                 LEFT JOIN POSESTCONS P ON (IT.CDFILIAL  = P.CDFILIAL)  AND
                                           (IT.CDPRODUTO = P.CDPRODUTO)
                 LEFT JOIN ESTOSEGU PF ON (IT.CDFILIAL  = PF.CDFILIAL)  AND
                                          (IT.CDPRODUTO = PF.CDPRODESTO)
                 LEFT JOIN  PARAMPRODMRP PP ON (IT.CDFILIAL  = PP.CDFILIAL)  AND
                                               (IT.CDPRODUTO = PP.CDPRODUTO)
         WHERE
            (PROCE.CDFILIAL     = IT.CDFILIAL)                       AND
            (PROCE.CDPRODUTO    = IT.CDPRODUTO)                      AND
            (PROCE.IDPROFABPADR = 'S')                               AND
            ((P.DTPOSESTCON = (SELECT MAX(PO.DTPOSESTCON)
                               FROM POSESTCONS PO
                               WHERE PO.CDFILIAL    = IT.CDFILIAL  AND
                                     PO.CDPRODUTO   = IT.CDPRODUTO AND
                                     PO.DTPOSESTCON <= (GETDATE() + 1)))    OR
              P.DTPOSESTCON IS NULL)                                    AND
            (IT.CDPRODUTO  = DT.CDPRODUTO)                              AND
            (IT.DTENTITPED = DT.DTENTITPED)
         ORDER BY
            DT.DTMINENTPED,
            IT.CDPRODUTO  

      OPEN C_PRODPLAN;
      FETCH NEXT FROM C_PRODPLAN INTO @C_PRODPLAN_CDPRODUTO, @C_PRODPLAN_IDPRODPROD, @C_PRODPLAN_DTMINENTPED,
                                      @C_PRODPLAN_QTDEESTO, @C_PRODPLAN_QTVENDAPCP, @C_PRODPLAN_QTESTOQMIN,
                                      @C_PRODPLAN_QTPADPROCFAB, @C_PRODPLAN_QTLOTEFABDIA,
                                      @C_PRODPLAN_QTPRODUCAO, @C_PRODPLAN_QTANDPROD, @C_PRODPLAN_QTTOTALPROD;

      WHILE (@@FETCH_STATUS = 0) BEGIN
         SELECT @V_NRORDENAOP = RIGHT(REPLICATE('0',10)+RTRIM(CONVERT(CHAR, CONVERT(NUMERIC, ISNULL(MAX(NRORDENAOP),'0')) + 1)),10)
         FROM CALEPRODMRP
         WHERE CDFILIAL = @P_CDFILIAL
           AND DTPROGPRODCP = @P_DTMRP;

         EXEC GERA_MRP_PRODUTO @C_PRODPLAN_QTANDPROD, @C_PRODPLAN_QTTOTALPROD, 'N', @P_CDFILIAL, @P_DTMRP, @C_PRODPLAN_CDPRODUTO, @C_PRODPLAN_IDPRODPROD,
                               @C_PRODPLAN_DTMINENTPED, @C_PRODPLAN_QTDEESTO, @C_PRODPLAN_QTESTOQMIN,
                               @C_PRODPLAN_QTPADPROCFAB, @C_PRODPLAN_QTLOTEFABDIA, @C_PRODPLAN_QTVENDAPCP,
                               @C_PRODPLAN_QTPRODUCAO, @P_DTINI, @P_DTFIN,
                               @P_CDPRODINI, @P_CDPRODFIN, @P_IDMRPPEDPREV, @P_IDESTOQUE, @P_IDCONSDEMA,
                               @V_NRORDENAOP, '0', @C_PRODPED_DTMINENTPED, @C_PRODPLAN_CDPRODUTO;

         FETCH NEXT FROM C_PRODPLAN INTO @C_PRODPLAN_CDPRODUTO, @C_PRODPLAN_IDPRODPROD, @C_PRODPLAN_DTMINENTPED,
                                         @C_PRODPLAN_QTDEESTO, @C_PRODPLAN_QTVENDAPCP, @C_PRODPLAN_QTESTOQMIN,
                                         @C_PRODPLAN_QTPADPROCFAB, @C_PRODPLAN_QTLOTEFABDIA,
                                         @C_PRODPLAN_QTPRODUCAO, @C_PRODPLAN_QTANDPROD, @C_PRODPLAN_QTTOTALPROD;
      END;
      CLOSE C_PRODPLAN;
      DEALLOCATE C_PRODPLAN;
   END;


   /* GUARDA OS PEDIDOS E AS SOLICITAES DE COMPRAS PLANEJADAS PARA QUE NO SEJAM PLANEJADAS NOVAMENTE*/

   IF ((CHARINDEX( 'P', @P_IDMRPPEDPREV) > 0)OR (CHARINDEX('O', @P_IDMRPPEDPREV) > 0) )BEGIN

      DECLARE C_REG CURSOR FOR
                   SELECT
                     ITEM.NRPEDIDO,
                     ITEM.NRITEMPED
                  FROM
                     PRODUTO PROD,
                     PEDIDO PED,
                     ITEMPEDIDO ITEM,
                     LISTA L
                  WHERE
                     (PROD.CDARVPROD BETWEEN @P_CDPRODINI AND @P_CDPRODFIN)    AND
                     (PED.CDFILIAL      = @P_CDFILIAL)                        AND
                     ((PED.IDSITPEDIDO ='2')OR(PED.IDSITPEDIDO = '6'))       AND
                     (ITEM.CDFILIAL     = PED.CDFILIAL)                      AND
                     (ITEM.NRPEDIDO     = PED.NRPEDIDO)                      AND
                     (L.NRLISTA       = @P_STLISTPEDIDO)                      AND
                     (ITEM.NRPEDIDO+'/'+ITEM.NRITEMPED = L.CDLISTA)        AND
                     (@P_STTODOPEDIDO <> 'T')                                 AND
                     (( CHARINDEX(PED.CDDEPTO, @P_CDDEPTO)>0)OR(@P_CDDEPTO IS NULL))AND
                     (((@P_ORCDE IS NULL)AND(@P_ORCATE IS NULL)AND(@P_LORC IS NULL))OR
                       (CHARINDEX( PED.NRORCAEVENTO, @P_LORC)>0)OR (PED.NRORCAEVENTO BETWEEN @P_ORCDE AND @P_ORCATE))AND
                     ((ITEM.IDSITITEMPED = '3') OR (ITEM.IDSITITEMPED = '7'))  AND
                     (ITEM.CDPRODUTO    = PROD.CDPRODUTO)                    AND
                     (CONVERT( DATETIME,ITEM.DTENTITPED, 103) BETWEEN  @P_DTINI AND @P_DTFIN) AND
                     ((CHARINDEX( 'P', @P_IDMRPPEDPREV) > 0)OR (CHARINDEX( 'O', @P_IDMRPPEDPREV) > 0) ) AND
                     (NOT EXISTS( SELECT CDFILIAL
                                  FROM PEDIDOP
                                  WHERE
                                     (CDFILIAL  = ITEM.CDFILIAL)AND
                                     (NRPEDIDO  = ITEM.NRPEDIDO)AND
                                     (NRITEMPED = ITEM.NRITEMPED)))
                  UNION

                  SELECT
                     ITEM.NRPEDIDO,
                     ITEM.NRITEMPED
                  FROM
                     PRODUTO PROD,
                     PEDIDO PED,
                     ITEMPEDIDO ITEM
                  WHERE
                     (PROD.CDARVPROD BETWEEN @P_CDPRODINI AND @P_CDPRODFIN)    AND
                     (PED.CDFILIAL      = @P_CDFILIAL)                        AND
                     ((PED.IDSITPEDIDO ='2')OR(PED.IDSITPEDIDO = '6'))       AND
                     (ITEM.CDFILIAL     = PED.CDFILIAL)                      AND
                     (ITEM.NRPEDIDO     = PED.NRPEDIDO)                      AND
                     (@P_STTODOPEDIDO  = 'T')                                 AND

                     ((CHARINDEX(PED.CDDEPTO, @P_CDDEPTO)>0)OR(@P_CDDEPTO IS NULL))AND
                     (((@P_ORCDE IS NULL)AND(@P_ORCATE IS NULL)AND(@P_LORC IS NULL))OR
                       (CHARINDEX(  PED.NRORCAEVENTO, @P_LORC)>0)OR (PED.NRORCAEVENTO BETWEEN @P_ORCDE AND @P_ORCATE))AND
                     ((ITEM.IDSITITEMPED = '3') OR (ITEM.IDSITITEMPED = '7'))  AND
                     (ITEM.CDPRODUTO    = PROD.CDPRODUTO)                    AND
                     (CONVERT( DATETIME ,ITEM.DTENTITPED, 103) BETWEEN  @P_DTINI AND @P_DTFIN) AND
                     ((CHARINDEX('P', @P_IDMRPPEDPREV) > 0)OR (CHARINDEX( 'O', @P_IDMRPPEDPREV) > 0) ) AND
                     (NOT EXISTS( SELECT CDFILIAL
                                  FROM PEDIDOP
                                  WHERE
                                     (CDFILIAL  = ITEM.CDFILIAL)AND
                                     (NRPEDIDO  = ITEM.NRPEDIDO)AND
                                     (NRITEMPED = ITEM.NRITEMPED)));

      OPEN C_REG;
      FETCH NEXT FROM C_REG INTO  @C_REG_NRPEDIDO, @C_REG_NRITEMPED;

      WHILE (@@FETCH_STATUS = 0) BEGIN
         INSERT INTO PEDIDOMRP( CDFILIAL, DTHRPEDIDOMRP, NRPEDIDO, NRITEMPED)
                VALUES( @P_CDFILIAL, @P_DTMRP, @C_REG_NRPEDIDO, @C_REG_NRITEMPED);
         FETCH NEXT FROM C_REG INTO  @C_REG_NRPEDIDO, @C_REG_NRITEMPED;
      END;
      CLOSE C_REG;
      DEALLOCATE C_REG;
   END;

   IF (CHARINDEX( 'S', @P_IDMRPPEDPREV) > 0)BEGIN
      DECLARE C_REG CURSOR FOR
                     SELECT
                        I.CDFILSOLI, I.CDFILLAN, I.NRSOLICMP, I.CDPRODUTO, I.DTPROGENTR
                     FROM SOLICITA SO, ITEMSOLI I, PRODUTO P, LISTA L, PARAMPRODMRP PA,
                          (SELECT CDFILIAL
                             FROM FILIAL
                            WHERE
                               ((CHARINDEX( CDFILIAL, @P_STLISTAFILI)>0)OR (@P_STTFILISOLI = 'T'))) F
                     WHERE
                           (SO.IDBAIXASOL IS NULL)
                       AND (@P_STTSOLICITA IS NULL)
                       AND (SO.CDFILLANCA = I.CDFILLAN)
                       AND (SO.CDFILSOLI  = I.CDFILSOLI)
                       AND (SO.NRSOLICMP  = I.NRSOLICMP)
                       AND (CDFILIALPED IS NULL)
                       AND (NRPEDIDO    IS NULL)
                       AND  I.CDPRODUTO = P.CDPRODUTO
                       AND (P.CDARVPROD BETWEEN @P_CDPRODINI AND @P_CDPRODFIN )
                       AND (IDSOLIAPROV = 'S')
                       AND DTPROGENTR BETWEEN @P_DTINI AND @P_DTFIN
                       AND (L.NRLISTA   = @P_STLISTASOLI)
                       AND (I.CDFILLAN+I.CDFILSOLI+I.NRSOLICMP = L.CDLISTA)
                       AND ( I.CDFILSOLI = F.CDFILIAL)
                       AND (@P_CDFILIAL   = PA.CDFILIAL)
                       AND (I.CDPRODUTO  = PA.CDPRODUTO)
                       AND (CHARINDEX( 'S', @P_IDMRPPEDPREV) > 0)
                       AND ( NOT EXISTS( SELECT NRSOLICMP
                                           FROM SOLICITAOP
                                          WHERE
                                             CDFILSOLI = I.CDFILSOLI AND
                                             CDFILLAN  = I.CDFILLAN AND
                                             NRSOLICMP = I.NRSOLICMP AND
                                             CDPRODUTO = I.CDPRODUTO AND
                                             DTPROGENTR = I.DTPROGENTR))


                     UNION

                     SELECT
                        I.CDFILSOLI, I.CDFILLAN, I.NRSOLICMP, I.CDPRODUTO, I.DTPROGENTR
                     FROM SOLICITA SO, ITEMSOLI I, PRODUTO P, PARAMPRODMRP PA,
                          (SELECT CDFILIAL
                             FROM FILIAL
                            WHERE
                               ((CHARINDEX(CDFILIAL, @P_STLISTAFILI)>0)OR (@P_STTFILISOLI = 'T'))) F
                     WHERE
                           (SO.IDBAIXASOL IS NULL)
                       AND (@P_STTSOLICITA = 'T')
                       AND (SO.CDFILLANCA = I.CDFILLAN)
                       AND (SO.CDFILSOLI  = I.CDFILSOLI)
                       AND (SO.NRSOLICMP  = I.NRSOLICMP)
                       AND (CDFILIALPED IS NULL)
                       AND (NRPEDIDO    IS NULL)
                       AND  I.CDPRODUTO = P.CDPRODUTO
                       AND (P.CDARVPROD BETWEEN @P_CDPRODINI AND @P_CDPRODFIN )
                       AND (IDSOLIAPROV = 'S')
                       AND DTPROGENTR BETWEEN @P_DTINI AND @P_DTFIN
                       AND (I.CDFILSOLI = F.CDFILIAL)
                       AND (@P_CDFILIAL   = PA.CDFILIAL)
                       AND (I.CDPRODUTO  = PA.CDPRODUTO)
                       AND (CHARINDEX('S', @P_IDMRPPEDPREV) > 0)
                       AND ( NOT EXISTS( SELECT NRSOLICMP
                                           FROM SOLICITAOP
                                          WHERE
                                             CDFILSOLI = I.CDFILSOLI AND
                                             CDFILLAN  = I.CDFILLAN AND
                                             NRSOLICMP = I.NRSOLICMP AND
                                             CDPRODUTO = I.CDPRODUTO AND
                                             DTPROGENTR = I.DTPROGENTR));


      OPEN C_REG;
      FETCH NEXT FROM C_REG INTO @C_REG_CDFILSOLI, @C_REG_CDFILLAN, @C_REG_NRSOLICMP, @C_REG_CDPRODUTO, @C_REG_DTPROGENTR;

      WHILE (@@FETCH_STATUS = 0) BEGIN
         INSERT INTO SOLICITAMRP( CDFILSOLI, CDFILLAN, NRSOLICMP, CDPRODUTO, DTPROGENTR, DTHRSOLICITAMRP, CDFILIAL)
                          VALUES( @C_REG_CDFILSOLI, @C_REG_CDFILLAN, @C_REG_NRSOLICMP, @C_REG_CDPRODUTO, @C_REG_DTPROGENTR, @P_DTMRP, @P_CDFILIAL);

         FETCH NEXT FROM C_REG INTO @C_REG_CDFILSOLI, @C_REG_CDFILLAN, @C_REG_NRSOLICMP, @C_REG_CDPRODUTO, @C_REG_DTPROGENTR;
      END;
      CLOSE C_REG;
      DEALLOCATE C_REG;
   END;

   /* Gera Matrias Primas */
   DECLARE C_MATEPRIMA CURSOR FOR
      SELECT
         INS.CDPRODINSUMO,
         CONVERT(DATETIME, CONVERT(VARCHAR, PRG.DTCONCPROMRP, 103), 103)  AS DTENTPROD,
         ISNULL(PRF.QTESTOSEGU, 0) AS QTESTOQMIN,
         SUM(TEKNISA.QTPREVI_MATPRIMA(@P_CDFILIAL, PRG.CDPRODUTO, INS.CDPROCFABRIC, INS.CDETAPAPROD, INS.CDPRODINSUMO, PRG.QTAFABRICMRP, 'S')) AS QTMATEPRIMA
      FROM
         PRODUTO PROD,
         PROCFABRICA PF,
         (SELECT
             CDPRODUTO,
             DTCONCPROMRP,
             QTAFABRICMRP
          FROM
             CALEPRODMRP
          WHERE
             (CDFILIAL     = @P_CDFILIAL) AND
             (DTPROGPRODCP = @P_DTMRP)
         ) PRG,
         INSUMETAPA INS LEFT JOIN POSESTCONS P ON (INS.CDPRODINSUMO = P.CDPRODUTO) AND
                                                  (P.CDFILIAL       = @P_CDFILIAL)
                        LEFT JOIN ESTOSEGU PRF ON (PRF.CDFILIAL     = INS.CDFILIAL)      AND
                                                  (PRF.CDPRODESTO   = INS.CDPRODINSUMO)
      WHERE
         (PROD.CDPRODUTO   = PRG.CDPRODUTO)    AND
         (PF.CDFILIAL      = @P_CDFILIAL)      AND
         (PF.CDPRODUTO     = PROD.CDPRODUTO)   AND
         (PF.IDPROFABPADR  = 'S')              AND
         (INS.CDFILIAL     = PF.CDFILIAL)      AND
         (INS.CDPRODUTO    = PF.CDPRODUTO)     AND
         (INS.CDPROCFABRIC = PF.CDPROCFABRIC)  AND
         (ISNULL(INS.IDSEMIACETAP,'N') <> 'S') AND
         ((P.DTPOSESTCON   = (SELECT MAX(PO.DTPOSESTCON)
                              FROM POSESTCONS PO
                              WHERE PO.CDFILIAL    = P.CDFILIAL        AND
                                    PO.CDPRODUTO   = INS.CDPRODINSUMO  AND
                                    PO.DTPOSESTCON <= (GETDATE() + 1))) OR
           P.DTPOSESTCON IS NULL)
      GROUP BY
         INS.CDPRODINSUMO,
         CONVERT(DATETIME, CONVERT(VARCHAR, PRG.DTCONCPROMRP, 103), 103),
         ISNULL(PRF.QTESTOSEGU, 0)
      ORDER BY
         CONVERT(DATETIME, CONVERT(VARCHAR, PRG.DTCONCPROMRP, 103), 103),
         INS.CDPRODINSUMO

   OPEN C_MATEPRIMA;
   FETCH NEXT FROM C_MATEPRIMA INTO @C_MATEPRIMA_CDPRODINSUMO, @C_MATEPRIMA_DTENTPROD,
                                    @C_MATEPRIMA_QTESTOQMIN, @C_MATEPRIMA_QTMATEPRIMA;

   WHILE (@@FETCH_STATUS = 0) BEGIN
      SELECT
         @V_QTCOMPRAS = ISNULL(SUM(QTPROGENTAF - ISNULL(QTEFETENTRAF,0)),0)
      FROM
         ITEMAUTF
      WHERE
         (CDFILDEST = @P_CDFILIAL)                   AND
         (CDPRODUTO = @C_MATEPRIMA_CDPRODINSUMO)     AND
         ((QTPROGENTAF - ISNULL(QTEFETENTRAF,0)) > 0);

      INSERT INTO PREMATPPROGP
         (CDFILIAL, DTPROGPRODMP, CDPRODPREVMP, DTUTILPROD,
          QTEMINMATPRI, QTPREVMATPRI, QTCOMPMATPRI)
      VALUES
         (@P_CDFILIAL, @P_DTMRP, @C_MATEPRIMA_CDPRODINSUMO, @C_MATEPRIMA_DTENTPROD,
          @C_MATEPRIMA_QTESTOQMIN, @C_MATEPRIMA_QTMATEPRIMA, @V_QTCOMPRAS);

      FETCH NEXT FROM C_MATEPRIMA INTO @C_MATEPRIMA_CDPRODINSUMO, @C_MATEPRIMA_DTENTPROD,
                                       @C_MATEPRIMA_QTESTOQMIN, @C_MATEPRIMA_QTMATEPRIMA;
   END;
   CLOSE C_MATEPRIMA;
   DEALLOCATE C_MATEPRIMA;


   IF (@P_IDGERSEMIACA = 'S') BEGIN
      DECLARE C_PROD CURSOR FOR
                     SELECT DISTINCT CDPRODUTO
                     FROM
                        PROGPRODU
                     WHERE
                        DTPROGPRODU = @P_DTMRP AND
                        CDFILIAL = @P_CDFILIAL

      OPEN C_PROD;
      FETCH NEXT FROM C_PROD INTO @C_PRODPED_CDPRODUTO

      WHILE(@@FETCH_STATUS = 0) BEGIN

         DELETE FROM CALEPRODMRP
         WHERE
            CDFILIAL = @P_CDFILIAL AND
            DTPROGPRODCP = @P_DTMRP AND
            CDPRODUTO    = @C_PRODPED_CDPRODUTO AND
            NOT EXISTS
                  (SELECT
                        CDPRODUTO
                   FROM  INSUMETAPA
                   WHERE
                      CDFILIAL  = @P_CDFILIAL AND
                      CDPRODINSUMO = @C_PRODPED_CDPRODUTO AND
                      IDSEMIACETAP = 'S');


         DELETE FROM PROGPRODU
         WHERE
            CDFILIAL    = @P_CDFILIAL AND
            DTPROGPRODU = @P_DTMRP    AND
            CDPRODUTO   = @C_PRODPED_CDPRODUTO AND
            NOT EXISTS
                  (SELECT
                        CDPRODUTO
                   FROM  INSUMETAPA
                   WHERE
                      CDFILIAL = @P_CDFILIAL AND
                      CDPRODINSUMO = @C_PRODPED_CDPRODUTO AND
                      IDSEMIACETAP = 'S');
         FETCH NEXT FROM C_PROD INTO @C_PRODPED_CDPRODUTO

      END;
      CLOSE C_PROD;
      DEALLOCATE C_PROD;
   END
   ELSE IF (@P_IDGERSEMIACA = 'P') BEGIN
      DECLARE C_PROD CURSOR FOR
                     SELECT DISTINCT CDPRODUTO
                     FROM
                        PROGPRODU
                     WHERE
                        DTPROGPRODU = @P_DTMRP AND
                        CDFILIAL = @P_CDFILIAL

      OPEN C_PROD;
      FETCH NEXT FROM C_PROD INTO @C_PRODPED_CDPRODUTO

      WHILE(@@FETCH_STATUS = 0) BEGIN

         DELETE FROM CALEPRODMRP
         WHERE
            CDFILIAL = @P_CDFILIAL AND
            DTPROGPRODCP = @P_DTMRP AND
            CDPRODUTO    = @C_PRODPED_CDPRODUTO AND
            EXISTS
                  (SELECT
                        CDPRODUTO
                   FROM  INSUMETAPA
                   WHERE
                      CDFILIAL  = @P_CDFILIAL AND
                      CDPRODINSUMO = @C_PRODPED_CDPRODUTO AND
                      IDSEMIACETAP = 'S');


         DELETE FROM PROGPRODU
         WHERE
            CDFILIAL    = @P_CDFILIAL AND
            DTPROGPRODU = @P_DTMRP    AND
            CDPRODUTO   = @C_PRODPED_CDPRODUTO AND
            EXISTS
                  (SELECT
                        CDPRODUTO
                   FROM  INSUMETAPA
                   WHERE
                      CDFILIAL = @P_CDFILIAL AND
                      CDPRODINSUMO = @C_PRODPED_CDPRODUTO AND
                      IDSEMIACETAP = 'S');
         FETCH NEXT FROM C_PROD INTO @C_PRODPED_CDPRODUTO

      END;
      CLOSE C_PROD;
      DEALLOCATE C_PROD;
   END;

END
GO

-- VERSAO --
EXEC ATUVEROBJ 'GERA_PLANEJ_MRP','5.05.016';
GO

